#!/bin/sh

BACKUP_DIR=/media/card/backup

date_iso8601() {
    date +"%FT%T%z"
}

parse_date_only()
{
    # busybox refuses to parse date only also requires a timestamp
    date -d "$1 12:00:00" +"%s"
}

ls_backups()
{
    for bak in $BACKUP_DIR/*; do
        if [ ! -d $bak ]; then
            continue
        fi
        dat="$(basename $bak)"
        if ! parse_date_only $dat > /dev/null 2>&1; then
            echo "unexpected backup folder: $bak" >&2
            continue
        fi
        echo $dat
    done
}

older_2years()
{
    THR_DATE=$(expr $(date "+%Y") - 2)-$(date "+%m-%d")
    THR_SEC=$(parse_date_only $THR_DATE)

    while read bak; do
        SEC=$(parse_date_only "$bak")
        if [ $SEC -gt $THR_SEC ]; then
            # keep
            continue
        fi
        echo $bak
    done
}

discard_backup() {
    while read bak; do
        echo remove $bak
        rm -rf $BACKUP_DIR/$bak
    done
}

if [ -n "$UNITTEST" ]; then
    return
fi

echo "$(date_iso8601) Discard old backups"
ls_backups | older_2years | discard_backup
