#!/bin/sh

set -e

UNITTEST=1
. ./sdo-disable-logs.sh # runs with dash

target()
{
    grep -q ARM926EJ /proc/cpuinfo
}

if ! target; then
    echo "Tests must be run on target"
    exit 1
fi

test_passed()
{
    echo "PASSED"
}

check()
{
    fct=$1
    exp_ret=$2
    exp_output=$3
    ret=0
    # eval runs in a separate shell, it's environment
    # is not visible from this shell
    output=`eval $fct 2>&1` || ret=$?
    [ $ret -eq $exp_ret ] || die "$fct: return code mismatch [$exp_ret != $ret]"

    if [ -z "$exp_output" ]; then
        return
    fi

    [ "$output" = "$exp_output" ] || \
        die "$fct: output mismatch [\'$exp_output\' != \'$output\']"
}

echo "######################################"
echo "TEST: runit_service"
echo "######################################"
check "runit_service dss" 0 ""
check "runit_service runit.sh" 1 ""
test_passed

echo "######################################"
echo "TEST: lookup_logdir"
echo "######################################"
for D in $MANAGED_DAEMONS; do
    check "lookup_logdir $d" 0 "$SDO_ROOT_DIR/var/log/$d"
    check "test -d /var/log/$d" 0 ""
done
check "lookup_logdir sdcard-overlay" 0 "$SDO_ROOT_DIR/var/log/sdcard"
check "test -d /var/log/sdcard" 0 ""
test_passed

echo "######################################"
echo "TEST: MANAGED_DAEMONS init.d/runit detection"
echo "######################################"
for D in $MANAGED_DAEMONS; do
    if runit_service $D; then
        continue
    fi
    if initd_service $D; then
        continue
    fi
    if [ $D = "dss-addons" ]; then
        continue
    fi
    echo "Unknown managed daemon $D"
done
test_passed

echo "######################################"
echo "TEST: disable svlogd MANAGED_DAEMONS"
echo "######################################"
for D in $MANAGED_DAEMONS; do
    if ! is_runit_svlogd $D; then
        echo "not runit: $D... skip"
        continue
    fi

    local LOGDIR="$(lookup_logdir $D)"
    LOGDIR="${LOGDIR#$ROOTFS}"

    echo
    echo "daemon under test:$D svlogd:$LOGDIR"
    echo

    local PID="$(fuser "$LOGDIR")"
    echo "-sec: processes with open fd in $LOGDIR: <$PID>"

    runit_svlogd $D stop

    for i in $(seq 0 180); do
        local PID="$(fuser "$LOGDIR")"
        echo "${i} sec: processes with open fd in $LOGDIR: <$PID>"
        if [ -z "$PID" ]; then
            break;
        fi
        sleep 1
    done
    check "fuser ${LOGDIR}" 1 # -> no more open fd

    echo
    echo "restarting daemon:$D svlogd:$LOGDIR"
    runit_svlogd $D start

    for i in $(seq 0 180); do
        local PID="$(fuser "$LOGDIR")"
        echo "${i} sec: processes with open fd in $LOGDIR: <$PID>"
        if [ -n "$PID" ]; then
            break;
        fi
        sleep 1
    done
    check "fuser $LOGDIR" 0 # -> has open fd
done
test_passed
